-- Author: DanioMods
-- Date: 20.07.2021
-- Version: Farming Simulator 19, 1.0.0.0
-- Copyright (C): DanioMods, All Rights Reserved
-- Specialization for grain augers

GrainAuger = {}

function GrainAuger.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Dischargeable, specializations)
end

function GrainAuger.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getCanDischargeToGround", GrainAuger.getCanDischargeToGround)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getCanDischargeToObject", GrainAuger.getCanDischargeToObject)
end

function GrainAuger:getCanDischargeToGround(superFunc, dischargeNode)
	if dischargeNode == nil then
		return false
	end
	
	if not dischargeNode.dischargeHitTerrain then
		return false
	end
	
	if self:getFillUnitFillLevel(dischargeNode.fillUnitIndex) > 0 then
		local fillTypeIndex = self:getDischargeFillType(dischargeNode)

		if not DensityMapHeightUtil.getCanTipToGround(fillTypeIndex) then
			return false
		end
	end
	
	if not self:getCanDischargeToLand(dischargeNode) then
		return false
	end
	
	if not self:getCanDischargeAtPosition(dischargeNode) then
		return false
	end
	
	local isTurnedOn = self:getIsTurnedOn()
	local specD = self.spec_dischargeable
	
	if not isTurnedOn then
		if specD.currentDischargeState ~= Dischargeable.DISCHARGE_STATE_OFF then
			self:setDischargeState(Dischargeable.DISCHARGE_STATE_OFF, true)
		end
		
		return false
	end
	
	return true
end

function GrainAuger:getCanDischargeToObject(superFunc, dischargeNode)
	if dischargeNode == nil then
		return false
	end
	
	local object = dischargeNode.dischargeObject
	
	if object == nil then
		return false
	end
	
	local fillType = self:getDischargeFillType(dischargeNode)
	
	if not object:getFillUnitSupportsFillType(dischargeNode.dischargeFillUnitIndex, fillType) then
		return false
	end
	
	local allowFillType = object:getFillUnitAllowsFillType(dischargeNode.dischargeFillUnitIndex, fillType)

	if not allowFillType then
		return false
	end
	
	if object.getFillUnitFreeCapacity ~= nil and object:getFillUnitFreeCapacity(dischargeNode.dischargeFillUnitIndex, fillType, self:getActiveFarm()) <= 0 then
		return false
	end
	
	if object.getIsFillAllowedFromFarm ~= nil and not object:getIsFillAllowedFromFarm(self:getActiveFarm()) then
		return false
	end
	
	if self.getMountObject ~= nil then
		local mounter = self:getDynamicMountObject() or self:getMountObject()

		if mounter ~= nil and not g_currentMission.accessHandler:canFarmAccess(mounter:getActiveFarm(), self, true) then
			return false
		end
	end
	
	local isTurnedOn = self:getIsTurnedOn()
	
	if not isTurnedOn then
		return false
	end
	
	return true
end